print("Helicopter prototype loading...")
inherit("AbstractObject") -- not actually necessary, but good form

function loadCommon(self)
  local h3d = TopObjects.h3d
  self.rootMat = h3d:AddResource(h3d.H3DResTypes.Material, "models/smoothbox/Material.material.xml", 0)
  self.topRotorModel = h3d:AddResource(h3d.H3DResTypes.SceneGraph, "models/heli/rotor.scene.xml", 0)
  self.heliModel = h3d:AddResource(h3d.H3DResTypes.SceneGraph, "models/heli/helibla.scene.xml", 0)
  --self.sideRotorModel = h3d:AddResource(h3d.H3DResTypes.SceneGraph, "models/heli/rotor.scene.xml", 0)
  h3d:LoadResourcesFromDisk()
  resourcesLoaded = true
end

function init(self)
  if not resourcesLoaded then self:loadCommon() end
  nextId = (nextId or 0) + 1
  self.heliId = nextId
  self.name = "heli" .. self.heliId
  local h3d = TopObjects.h3d
  -- we create a group node so that we can attach a camera without
  -- worrying about scaling issues
  self.baseNode = h3d:AddGroupNode(h3d.RootNode, self.name .. "_base")

  -- create the rotor group nodes
  self.topGN = h3d:AddGroupNode(self.baseNode, self.name .. "_topGN")
  self.sideGN = h3d:AddGroupNode(self.baseNode, self.name .. "_sideGN")
  self.trp = {-1.1,-0.6,0,0,0,0,1,1,1}
  self.srp = {-4.4,0.1,-0.9,90,0,0,1,1,1}
  self.tweakMultipliers = {1,1,1,50,50,50,1,1,1}
  h3d:SetNodeTransform(self.topGN, _G.unpack(self.trp))
  h3d:SetNodeTransform(self.sideGN, _G.unpack(self.srp))

  -- create rotors
  self.topRotor = h3d:AddNodes(self.topGN, self.topRotorModel)
  --self.sideRotor = h3d:AddNodes(self.sideGN, self.sideRotorModel)

  -- create body
  self.body = h3d:AddNodes(self.baseNode, self.heliModel)
  h3d:SetNodeTransform(self.body, -1,0,0, 0,0,0, 1, 1, 1)

  -- set rotor velocities
  self.trv = 360.0 * 4.0725
  self.srv = 360.0 * 4.0615
  self.tra = 0
  self.sra = 0

  self.dtmult = 1

  h3d:DumpMessages()

end

function setDtMult(self, mult)
  self.dtmult = mult
end

function setPosition(self, pos, rot)
  local h3d = TopObjects.h3d
  if #rot > 3 then
    -- assume quaternion
    h3d:SetNodeTransformQuaternion(self.baseNode, pos[1], pos[2], pos[3], rot[1], rot[2], rot[3], rot[4], 1, 1, 1)
  else
    h3d:SetNodeTransform(self.baseNode, pos[1], pos[2], pos[3], rot[1], rot[2], rot[3], 1, 1, 1)
  end
end

function printList(lst)
  ret = ""
  for i = 1,#lst do
    ret = ret .. lst[i] .. ", "
  end
  return ret
end

function updateRotors(self, dt)
  local h3d = TopObjects.h3d
  self.tra = self.tra + self.trv * dt
  self.sra = self.sra + self.srv * dt
  if self.tra > 360 then self.tra = self.tra - 360 end
  if self.sra > 360 then self.sra = self.sra - 360 end
  h3d:SetNodeTransform(self.topRotor, 0,0,0, 0,self.tra,0, 1, 1, 1)
  --h3d:SetNodeTransform(self.sideRotor, 0,0,0, 0,self.sra,0, 0.1, 0.1, 0.1)
end

function tweakTopRotorPos(self, idx, amt)
  local h3d = TopObjects.h3d
  self.trp[idx] = self.trp[idx] + (amt * self.tweakMultipliers[idx])
  h3d:SetNodeTransform(self.topGN, _G.unpack(self.trp))
  print("curpos: " .. printList(self.trp))
end

function tweakSideRotorPos(self, idx, amt)
  local h3d = TopObjects.h3d
  self.srp[idx] = self.srp[idx] + (amt * self.tweakMultipliers[idx])
  --h3d:SetNodeTransform(self.sideGN, _G.unpack(self.srp))
  print("curpos: " .. printList(self.srp))
end

function update(self, dt)
  self:updateRotors(dt * self.dtmult)
end