print("Test master prototype loading...")
print("Trying to inherit...")
inherit("AbstractObject")
print("Done inheriting...")

io = _G.io
table = _G.table
math = _G.math

-- random comment

function toDeg(rad)
  return rad / math.pi * 180.0
end

function setuph3d(self)
  print("Setting up h3d")
  local h3d = TopObjects.h3d
  self.sw = 800 --320
  self.sh = 600 --240
  h3d:InitGraphics(self.sw, self.sh, false)
  print("Adding resources!")
  self.pipeRes = h3d:AddResource(h3d.H3DResTypes.Pipeline, "pipelines/forward.pipeline.xml", 0)
  self.blurPipeRes = h3d:AddResource(h3d.H3DResTypes.Pipeline, "pipelines/blur.pipeline.xml", 0)
  self.platRes = h3d:AddResource(h3d.H3DResTypes.SceneGraph, "models/platform/platform.scene.xml", 0)
  self.skyboxRes = h3d:AddResource(h3d.H3DResTypes.SceneGraph, "models/skybox/skybox.scene.xml", 0)
  self.lightRes = h3d:AddResource(h3d.H3DResTypes.Material, "materials/light.material.xml", 0)
  print("Loading resources!")
  h3d:LoadResourcesFromDisk()
  print("Adding lights and stuff!")
  self.skybox = h3d:AddNodes(h3d.RootNode, self.skyboxRes)
  self.platform = h3d:AddNodes(h3d.RootNode, self.platRes)
  print("Added model! Adding light!")
  self.light = h3d:AddLightNode(h3d.RootNode, "Light1", self.lightRes, "LIGHTING", "SHADOWMAP")
  print("Setting transforms and whatnot!")
  print("Light: " .. h3d:AsInt(self.light))
  h3d:SetNodeTransform(self.skybox, 0, 0, 0, 0, 0, 0, 210, 210, 210)
  h3d:SetNodeTransform(self.platform, 0, 0, 0, 0, 0, 0, 2.0, 2.0, 2.0)
  h3d:SetNodeTransform(self.light, 0, 125, 250, -30, 0, 0, 1, 1, 1)
  h3d:SetNodeParamF(self.light, h3d.H3DLight.RadiusF, 0, 5000.0)
  h3d:SetNodeParamF(self.light, h3d.H3DLight.FovF, 0, 180)
  h3d:SetNodeParamI(self.light, h3d.H3DLight.ShadowMapCountI, 3)
  h3d:SetNodeParamF(self.light, h3d.H3DLight.ShadowSplitLambdaF, 0, 0.9)
  h3d:SetNodeParamF(self.light, h3d.H3DLight.ShadowMapBiasF, 0, 0.0015)
  h3d:SetNodeParamF(self.light, h3d.H3DLight.ColorF3, 0, 0.9)
  h3d:SetNodeParamF(self.light, h3d.H3DLight.ColorF3, 1, 0.9)
  h3d:SetNodeParamF(self.light, h3d.H3DLight.ColorF3, 2, 0.75)
  print("Camera!")

  self.normcam = h3d:AddCameraNode(h3d.RootNode, "Camera", self.pipeRes)
  self.blurcam = h3d:AddCameraNode(h3d.RootNode, "BlurCamera", self.blurPipeRes)
  self.cam = self.normcam

  h3d:EnableAutoUpdate(self.cam)
  h3d:DumpMessages()
end

function dispatch_message(self, msg)
  --print("message!")
  for k,v in _G.pairs(msg) do
    print(k .. ": " .. (v or "nil"))
  end
end

function mouseMoveEvent(self, msg)
  local curvel = 0;
  local math = _G.math

  --print("Mouse move event!")
  --print("x: " .. msg.x .. ", y: " .. msg.y)
  --print("dx: " .. msg.dx .. ", dy: " .. msg.dy)

  if _G.math.abs(msg.dx) < 50 and _G.math.abs(msg.dy) < 50 and (self.mbuttondown or 0) > 0 then
    if self.mbuttonid == 1 then
		self.ry = self.ry + msg.dx / 100 * 20
		self.rx = self.rx + msg.dy / 100 * 20
		if self.rx > 90 then self.rx = 90 end
		if self.rx < -90 then self.rx = -90 end
	elseif self.mbuttonid == 2 then
	  if msg.dy < 0 then
	    curvel = math.abs(msg.dy) / 20
		self.x = self.x - (math.sin(degToRad(self.ry)) * math.cos(-1 * degToRad(self.rx))) * curvel
		self.y = self.y - math.sin(-1 * degToRad(self.rx)) * curvel
		self.z = self.z - math.cos(degToRad(self.ry)) * math.cos(-1 * degToRad(self.rx)) * curvel
	  end
	  
	  if msg.dx > 0 then
	    curvel = math.abs(msg.dx) / 20
		self.x = self.x - math.cos(degToRad(self.ry)) * curvel
		self.z = self.z + math.sin(degToRad(self.ry)) * curvel
	  end

	  if msg.dy > 0 then
	    curvel = math.abs(msg.dy) / 20
		self.x = self.x + (math.sin(degToRad(self.ry)) * math.cos(-1 * degToRad(self.rx))) * curvel
		self.y = self.y + math.sin(-1 * degToRad(self.rx)) * curvel
		self.z = self.z + math.cos(degToRad(self.ry)) * math.cos(-1 * degToRad(self.rx)) * curvel
	  end

	  if msg.dx < 0 then
	    curvel = math.abs(msg.dx) / 20
		self.x = self.x + math.cos(degToRad(self.ry)) * curvel
		self.z = self.z - math.sin(degToRad(self.ry)) * curvel
	  end
	end
  end
end

function keyEvent(self, msg)
  local h3d = TopObjects.h3d
  self:dispatch_message(msg)
  self.keys[msg.keyName] = (msg.keyDown == 1)
  if msg.keyDown == 1 and msg.keyName == "F6" then
    self.uvFrame = TopObjects.h3d:GetScreenshot(self.frame)
  end
  if msg.keyDown == 1 and msg.keyName == "F10" then
    self.texFrame = TopObjects.h3d:GetScreenshot(self.frame)
  end
  local nval = _G.tonumber(msg.keyName)
  if msg.keyDown == 1 and nval ~= nil and nval >= 1 and nval <= 9 then
    print("Number hit: " .. nval)
    self.idx = nval
  end
  if msg.keyDown == 1 and msg.keyName == "KP_PLUS" then
    self.heli:tweakSideRotorPos(self.idx, 0.1)
  end
  if msg.keyDown == 1 and msg.keyName == "KP_MINUS" then
    self.heli:tweakSideRotorPos(self.idx, -0.1)
  end
  if msg.keyDown == 1 and msg.keyName == "F1" then
    self.cam = self.normcam
    h3d:EnableAutoUpdate(self.cam)
  end
  if msg.keyDown == 1 and msg.keyName == "F2" then
    self.cam = self.blurcam
    h3d:EnableAutoUpdate(self.cam)
  end
  if msg.keyDown == 1 and msg.keyName == "F3" then
    self:enableSubFrames(10)
  end
  if msg.keyDown == 1 and msg.keyName == "F4" then
    self:enableSubFrames(-1)
  end
  if msg.keyDown == 1 and msg.keyName == "F5" then
    self.x, self.y, self.z = 10, 10, 10
	self.rx, self.ry, self.rz = -25, 47, 0
  end
end

function mouseButtonEvent(self,msg)
  self:dispatch_message(msg)
  self.mbuttondown = msg.buttonDown
  self.mbuttonid = msg.buttonID
end

function enableSubFrames(self, nsub)
  local h3d = TopObjects.h3d
  if nsub <= 0 then
    self.subFramesEnabled = false
    h3d:SetSwapBufferInterval(0)
    self.heli:setDtMult(1.0)
  else
    h3d:SetSwapBufferInterval(nsub)
    self.subFramesEnabled = true
    self.nsub = nsub
    self.heli:setDtMult(1.0 / nsub)
  end
end

function init(self)
  print("Test master init!")
  math.randomseed(_G.os.time())

  self.keytrans = {}
  for i = 1,10 do
    self.keytrans[_G.tostring(i)] = i
  end

  local h3d_dummy = _G.Core.CreateTLO("Horde3d", "h3d")
  local bt_dummy = _G.Core.CreateTLO("PlBullet", "bullet")

  self:setuph3d()

  local dp = _G.Core.CreateTLO("Dispatch", "disp")
  local revel = _G.Core.CreateTLO("RevelEncoder", "Revel")
  dp:AddNamedHandler("event:mouseMove", "mouseMoveEvent", self)
  dp:AddNamedHandler("event:keyEvent", "keyEvent", self)
  dp:AddNamedHandler("event:mouseButton", "mouseButtonEvent", self)

  self.heli = _G.Core.CreateTLO("game.Helicopter", "theHeli")
  self.heli:init()
  self.heli:setPosition({0, 3, 0}, {0, 0, 0})

  self.timeleft = 10
  
  self.x, self.y, self.z = 10, 10, 10
  self.rx, self.ry, self.rz = -15, 47, 0
  
  self.keys = {}

  self.encoding = false
end

function grabFrame(self)
  self.frame = TopObjects.h3d:GetBBScreenshot(self.frame)
  if self.frame then
    print("Frame size: " .. self.frame:SizeX() .. " " .. self.frame:SizeY())
    local mx = self.frame:SizeX() / 2
    local my = self.frame:SizeY() / 2
    local r = self.frame:Get(mx, my, 0)
    local g = self.frame:Get(mx, my, 1)
    local b = self.frame:Get(mx, my, 2)
    print("Center rgb: " .. r .. " " .. g .. " " .. b)
  end
end

function degToRad(val)
  return (val / 180.0) * 3.1415926
end

function setCameraPosition(self, pos, rot)
  self.x, self.y, self.z = pos[1], pos[2], pos[3]
  self.rx, self.ry, self.rz = rot[1], rot[2], rot[3]
  --local h3d = TopObjects.h3d
  --if #rot > 3 then
    -- assume quaternion
  --  h3d:SetNodeTransform(self.baseNode, pos[1], pos[2], pos[3], rot[1], rot[2], rot[3], rot[4], 1, 1, 1)
  --else
  --  h3d:SetNodeTransform(self.baseNode, pos[1], pos[2], pos[3], rot[1], rot[2], rot[3], 1, 1, 1)
  --end
end

function doCameraMovement(self, dt)
  local curvel = dt * 30
  local math = _G.math

  if self.keys["w"] then
    self.x = self.x - (math.sin(degToRad(self.ry)) * math.cos(-1 * degToRad(self.rx))) * curvel
    self.y = self.y - math.sin(-1 * degToRad(self.rx)) * curvel
    self.z = self.z - math.cos(degToRad(self.ry)) * math.cos(-1 * degToRad(self.rx)) * curvel
  end
  
  if self.keys["a"] then
    self.x = self.x - math.cos(degToRad(self.ry)) * curvel
    self.z = self.z + math.sin(degToRad(self.ry)) * curvel
  end

  if self.keys["s"] then
    self.x = self.x + (math.sin(degToRad(self.ry)) * math.cos(-1 * degToRad(self.rx))) * curvel
    self.y = self.y + math.sin(-1 * degToRad(self.rx)) * curvel
    self.z = self.z + math.cos(degToRad(self.ry)) * math.cos(-1 * degToRad(self.rx)) * curvel
  end

  if self.keys["d"] then
    self.x = self.x + math.cos(degToRad(self.ry)) * curvel
    self.z = self.z - math.sin(degToRad(self.ry)) * curvel
  end

  if self.keys["SPACE"] then
    self.y = self.y + curvel
  end

  if self.keys["LCTRL"] then
    self.y = self.y - curvel
  end

  TopObjects.h3d:SetNodeTransform(self.cam, self.x, self.y, self.z, self.rx, self.ry, self.rz, 1, 1, 1)
end

function startEncoding(self, fn)
  if not self.encoding then
    TopObjects.Revel:StartEncoding(self.sw, self.sh, 6.0, 30, fn)
    self.encoding = true
  end
end

function stopEncoding(self)
  if self.encoding then
    TopObjects.Revel:FinalizeEncoding()
    self.encoding = false
  end
end

function update(self, dt)
  self:doCameraMovement(dt)

  if self.encoding then
    self.frame = TopObjects.h3d:GetScreenshot(self.frame)
    if self.frame ~= nil then
      TopObjects.Revel:EncodeFrame(self.frame)
    end
  end
end